/*
    Copyright (C) 2005-2012  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _NNTPGRAB_TYPES_H_
#define _NNTPGRAB_TYPES_H_

#include <sys/types.h>
#include <time.h>

#ifdef  __cplusplus
extern "C" {
#endif

#ifndef DONT_WRAP_GLIB

/* These declarations are actually from GLib, but to easy the development
 * of external frontends, they have been duplicated here to avoid linkage bloat */
#ifndef FALSE
#define FALSE   (0)
#endif

#ifndef TRUE
#define TRUE    (!FALSE)
#endif

typedef int                 ngboolean;
#ifdef WIN32
typedef long long           ngint64;
typedef unsigned long long  nguint64;
#else
typedef int64_t             ngint64;
typedef __uint64_t          nguint64;
#endif

typedef struct _NGList NGList;
typedef void NGRegex;

struct _NGList
{
  void *data;
  NGList *next;
  NGList *prev;
};

#define NG_CALLBACK(f)                   ((NGCallback) (f))
typedef void  (*NGCallback)              (void);

typedef void*                            NGType;
typedef struct _NGTypeInstance           NGTypeInstance;
typedef struct _NGTypeClass              NGTypeClass;

struct _NGTypeInstance
{
  /*< private >*/
  NGTypeClass *ng_class;
};

#  define _NG_TYPE_CIC(ip, gt, ct) \
    ((ct*) ng_type_check_instance_cast ((NGTypeInstance*) ip, gt))
#define NG_TYPE_CHECK_INSTANCE_CAST(instance, ng_type, c_type)    (_NG_TYPE_CIC ((instance), (ng_type), c_type))

#  define _NG_TYPE_CCC(cp, gt, ct) \
    ((ct*) ng_type_check_class_cast ((NGTypeClass*) cp, gt))
#define NG_TYPE_CHECK_CLASS_CAST(ng_class, ng_type, c_type)        (_NG_TYPE_CCC ((ng_class), (ng_type), c_type))

#  define _NG_TYPE_CIT(ip, gt)             (ng_type_check_instance_is_a ((NGTypeInstance*) ip, gt))
#define NG_TYPE_CHECK_INSTANCE_TYPE(instance, ng_type)            (_NG_TYPE_CIT ((instance), (ng_type)))

#  define _NG_TYPE_CCT(cp, gt)             (ng_type_check_class_is_a ((NGTypeClass*) cp, gt))
#define NG_TYPE_CHECK_CLASS_TYPE(ng_class, ng_type)                (_NG_TYPE_CCT ((ng_class), (ng_type)))

#define _NG_TYPE_IGC(ip, gt, ct)         ((ct*) (((NGTypeInstance*) ip)->ng_class))
#define NG_TYPE_INSTANCE_GET_CLASS(instance, ng_type, c_type)     (_NG_TYPE_IGC ((instance), (ng_type), c_type))

NGTypeInstance*   ng_type_check_instance_cast   (NGTypeInstance      *instance,
                                                 NGType               iface_type);

/* End of GLib declarations */

#else

#include <glib.h>
#include <glib-object.h>

#define NGList GList
#define NGRegex GRegex
#define NGCallback GCallback
#define NGType GType
#define ngboolean gboolean
#define ngint64 gint64
#define nguint64 guint64

#endif /* !DONT_WRAP_GLIB */

typedef enum {
    SERVER_PRIORITY_HIGH,
    SERVER_PRIORITY_NORMAL,
    SERVER_PRIORITY_LOW
} NGServerPriority;

/**
 * ConfigServer:
 * @servername:         The name of the server (this will be shown in the user interface)
 * @hostname:           The hostname of IP address of the server
 * @port:               The port on which a NNTP server is running on @hostname
 * @username:           The username which is required to authenticate (can be "" if no username is required)
 * @password:           The password which is required to authenticate (can be "" if no password is required)
 * @max_threads:        The maximum number of simultanous connections to this server
 * @priority:           Will be used to determine the order in which servers needs to be used if there are multiple servers configured
 * @send_group_command: Some usenet server require that the GROUP command is sent before downloading can start
 * @use_ssl:            When TRUE, a SSL connection will be set up to the server, if FALSE, a regular NNTP connection will be set up
 * @enabled:            Whether is server needs to be used
 */
typedef struct _config_server {
    char servername[128];
    char hostname[128];
    int port;
    char username[64];
    char password[64];
    int max_threads;
    NGServerPriority priority;
    ngboolean send_group_command;
    ngboolean use_ssl;
    ngboolean enabled;
} NGConfigServer;

/**
 * ConfigOpts:
 * @download_directory:                     The folder in which all downloads need to be saved
 * @temp_directory:                         The folder where all temporary downloaded parts will be saved
 * @enable_intelligent_par2_downloading:    If TRUE, PAR2 files will only be downloaded when needed. If FALSE, all files will be downloaded
 * @enable_par2_repair:                     Automatically repair files using PAR2 after a collection has been fully downloaded
 * @auto_import_directory:                  The folder which needs to be monitored for new .NZB files
 * @enable_auto_import:                     Whether the automatic import feature needs to be enabled (if TRUE, @auto_import_directory also needs to be set to a sane value)
 * @move_file_after_auto_import:            If TRUE, all automatically imported files will be moved to a folder named 'Imported' under @auto_import_directory
 * @enable_auto_unpack:                     Whether files needs to be unpacked automatically when a collection has been fully downloaded (and optionally repaired)
 * @enable_bandwidth_shaping:               Whether bandwidth shaping needs to be applied
 * @max_bandwidth:                          The maximum number of KB/sec which can be used for downloading if @enable_bandwidth_shaping is set
 * @enable_webserver:                       Whether the embedded webserver needs to be activated. Requires the JSON-RPC plugin to be loaded
 * @webserver_port:                         The port on which the embedded webserver needs to listen when @enable_webserver is set
 * @enable_logger:                          Whether automatic debug logging needs to be activated. Required the Logger plugin to be loaded. All logs will be saved to ~/.config/NNTPGrab/nntpgrab.log
 * @auto_remove_files_after_repair:         Automatically remove files after a successfull PAR2 repair
 * @auto_remove_files_after_unpack:         Automatically remove files after a successfull automatic unpack
 * @auto_remove_collections_after_download: Automatically remove collections from the download queue after a succesfull download
 */
typedef struct _config_opts {
    char download_directory[256];
    char temp_directory[256];
    ngboolean enable_intelligent_par2_downloading;
    ngboolean enable_par2_repair;
    char auto_import_directory[256];
    ngboolean enable_auto_import;
    ngboolean move_file_after_auto_import;
    ngboolean enable_auto_unpack;
    ngboolean enable_bandwidth_shaping;
    int max_bandwidth;
    ngboolean enable_webserver;
    int webserver_port;
    ngboolean enable_logger;
    ngboolean auto_remove_files_after_repair;
    ngboolean auto_remove_files_after_unpack;
    ngboolean auto_remove_collections_after_download;
} NGConfigOpts;

/**
 * ConfigGroupInfo:
 *
 * This structure is currently unused in NNTPGrab 0.6
 */
typedef struct _config_group_info {
    char newsgroup[256];
    ngboolean is_subscribed;
    ngboolean is_moderated;
    int num_articles;
    int start_article;
    int end_article;
    time_t last_checked;
} ConfigGroupInfo;

/**
 * NNTPGrabPart:
 * @message_id:     The message ID of the part (with the < and >)
 * @size:           The size of the part in bytes
 * @part_num:       The number of the part
 *
 * This structure is used for the functions nntpgrab_core_schedular_add_task_to_queue() and nntpgrab_schedular_add_task_to_queue()
 */
typedef struct _nntpgrab_part {
    char message_id[256];
    int size;
    int part_num;
} NNTPGrabPart;

/**
 * NNTPGrabFolder:
 * @folder:             The last element of the folder name
 * @has_subfolders:     Whether this folder has subfolders
 *
 * This structure is used for the functions nntpgrab_utils_get_folder_listing() and nntpgrab_config_get_avail_servers()
 */
typedef struct _nntpgrab_folder {
    char folder[2048];
    ngboolean has_subfolders;
} NNTPGrabFolder;

/**
 * NNTPGrabPluginInfo:
 * @name:               The name of the plugin
 * @version:            The version of the plugin
 * @author:             Who created the plugin
 * @url:                A link to the website of the plugin
 * @description:        A description indicated the purpose of the plugin
 * @is_loaded:          Whether the plugin is loaded at the moment
 * @is_persistent:      Unused in NNTPGrab 0.6
 *
 * This structure is used by the functions nntpgrab_core_plugins_get_plugin_info() and nntpgrab_plugins_get_plugin_info()
 */
typedef struct {
    char name[128];
    char version[64];
    char author[128];
    char url[256];
    char description[4096];

    ngboolean is_loaded;
    ngboolean is_persistent;
} NNTPGrabPluginInfo;

typedef enum {
    CONFIG_CHANGED_NOTHING,
    CONFIG_CHANGED_SERVER_ADDED,
    CONFIG_CHANGED_SERVER_DELETED,
    CONFIG_CHANGED_OPTS_CHANGED
} ConfigChangedFlag;

typedef enum {
    SCHEDULAR_STATE_RUNNING,
    SCHEDULAR_STATE_STOPPING,
    SCHEDULAR_STATE_STOPPED
} NGSchedularState;

typedef enum _nntp_disconnect_type {
    DISCONNECT_NORMAL,
    DISCONNECT_NO_SUCH_HOST,
    DISCONNECT_CONNECTION_REFUSED,
    DISCONNECT_TOO_MANY_CONNECTIONS,
    DISCONNECT_CONNECT_TIMEOUT,
    DISCONNECT_READ_ERROR,
    DISCONNECT_READ_TIMEOUT,
    DISCONNECT_WRITE_ERROR,
    DISCONNECT_IDLE_TIMEOUT,
    DISCONNECT_INVALID_MSG,
    DISCONNECT_LOGIN_FAILURE,
    DISCONNECT_ERROR_SSL_INITIALISE,
    DISCONNECT_UNEXPECTED
} NNTPDisconnectType;

typedef enum _nntp_file_type {
    NNTP_FILE_TYPE_UNKNOWN,
    NNTP_FILE_TYPE_RAR,
    NNTP_FILE_TYPE_PAR,
    NNTP_FILE_TYPE_PAR2,
    NNTP_FILE_TYPE_SFV,
    NNTP_FILE_TYPE_NFO,
    NNTP_FILE_TYPE_NZB,
    NNTP_FILE_TYPE_OTHER
} NNTPFileType;

typedef enum {
    TASK_STATE_WAITING_FOR_DOWNLOAD,
    TASK_STATE_DOWNLOADING,
    TASK_STATE_WAITING_FOR_DECODE,
    TASK_STATE_DECODING,
    TASK_STATE_FINISHED_COMPLETE,
    TASK_STATE_FINISHED_INCOMPLETE,
    TASK_STATE_FINISHED_NO_PARTS_AVAIL,
    TASK_STATE_SKIPPED
} NGTaskState;

/**
 * PAR2FileState:
 *
 * This is currently unused in NNTPGrab 0.6
 */
typedef enum _par2_file_state {
    PAR2_FILE_STATE_MISSING,
    PAR2_FILE_STATE_FOUND,
    PAR2_FILE_STATE_DAMAGED,
    PAR2_FILE_STATE_NO_NEW_BLOCKS_FOUND
} PAR2FileState;

typedef enum {
    NG_LOG_LEVEL_ALL = 0,
    NG_LOG_LEVEL_INFO,
    NG_LOG_LEVEL_WARNING,
    NG_LOG_LEVEL_ERROR,
    NG_LOG_LEVEL_FATAL,
    NG_LOG_LEVEL_DEBUG,
} NGLogLevel;

/**
 * ForeachCollectionFunc:
 * @collection_name:                    The name of the collection
 * @poster:               (allow-none): The poster of the collection. Can be NULL if multiple posters are involved
 * @total_size:                         The total size of the collection in bytes
 * @total_size_remaining:               The number of bytes of files in this collection which still need to be downloaded
 * @position:                           The position of this collection in the entire download queue
 * @user_data:                          User provided callback data
 */
typedef void (*ForeachCollectionFunc) (const char *collection_name, const char *poster, nguint64 total_size, nguint64 total_size_remaining, int position, void *user_data);

/**
 * ForeachFileFunc:
 * @collection_name:      The name of the collection
 * @subject:              The subject of the current file
 * @poster:               The poster of the file
 * @stamp:                The timestamp indicating when the file was posted
 * @file_size:            The size of the file in bytes
 * @file_size_remaining:  The number of bytes of this file which still need to be downloaded
 * @position:             The position of this file in the current collection
 * @num_parts_total:      The total number of parts of which this file consists
 * @num_parts_downloaded: The number of parts of this file which are already downloaded
 * @num_parts_failed:     The number of parts of this file which failed to download
 * @status:               The current status of this file
 * @user_data:            User provided callback data
 */
typedef void (*ForeachFileFunc) (const char *collection_name, const char *subject, const char *poster, ngint64 stamp, nguint64 file_size, nguint64 file_size_remaining, int position, int num_parts_total, int num_parts_downloaded, int num_parts_failed, NGTaskState status, const char *filename, void *user_data);

/**
 * ForeachGroupFunc:
 * @collection_name:      The name of the collection
 * @subject:              The subject of the current file
 * @group:                The group in which this file is posted
 * @user_data:            User provided callback data
 */
typedef void (*ForeachGroupFunc) (const char *collection_name, const char *subject, const char *group, void *user_data);

#ifdef  __cplusplus
}
#endif

#endif /* _NNTPGRAB_TYPES_H_ */
